/* #####################################################################
	This do-file accompanies a blog post on multilevel Tobit models
	written for Timberlake Consultants (timberlake.co.uk)
	by Robert Grant at BayesCamp (bayescamp.com)
##################################################################### */

/*
Generate some simulated data: rehabilitation for amputees learning to
use electrically-controlled prosthetic hands.
In these data, there are two training programmes being compared (variable: train)
and the participants (identified by the variable: id) are tracked over several weeks 
(variable: week). They have to complete a test by carrying out four tasks of 
increasing difficulty (variable: task), and they are timed on the tasks 
(variable: time), so smaller time is desirable, but there is a maximum time 
of 30 seconds, so if they have not completed it within 30 seconds, then time is 
recorded as 30. 
This is right-censoring of the dependent variable. We also have to account for the 
repeated-measures (economists may say "panel data") nature of the data in a 
multilevel model. There will be a random intercept by id, and a random slope for
week, and the study is principally interested in the interaction between week and 
train, because this would indicate a difference between training programmes in
their impact on the rate of recovery (slope relating week to time).
*/

clear all
set seed 455524
// true parameters:
global beta0 = 25
global beta1 = (-0.8)
global beta2 = (-1.1)
global sigma2_u = 5 // random intercept variance
global sigma_u = sqrt(${sigma2_u}) // random intercept SD
global sigma2_v = 0.5 // random slope variance
global sigma_v = sqrt(${sigma2_v}) // random slope SD 
global sigma2_eps = 1 // observation-level residual variance
global h = 30 // censoring threshold
macro dir // check these have been created as expected
// make the data:
set obs 100 
gen id=_n
gen train=(_n>50)
label define trainlab 0 "Old training" 1 "New training"
label values train trainlab
gen tempnorm1 = rnormal()
gen tempnorm2 = rnormal()
gen true_u = ${sigma_u} * (tempnorm1 + 0.5*tempnorm2)
gen true_v= (-1) * ${sigma_v} * (tempnorm1 - 0.5*tempnorm2) // neg. correlated
drop tempnorm1
drop tempnorm2
corr true_u true_v
scatter true_v true_u, name(check_random_effects, replace)
expand 5
bysort id: gen week=_n
expand 4
bysort id week: gen task=_n
gen true_residual = rnormal()
replace true_residual = true_residual * sqrt(${sigma2_eps})
// make the true outcome
gen y = ${beta0} + true_u + (${beta1}+true_v)*week + ${beta2}*train*week + ///
		2*task + true_residual
label var y "Response time (sec)"
// draw the trends over time
sort task train id week
forvalues i=1/4 {
	line y week if task==`i', connect(ascending) lcol(%15) by(train) ///
		title("Task `i'") yscale(range(0 40)) ylabel(0(10)40, grid) ///
		yline(30, lcol(red) lpattern(dash)) scheme(s1mono) ///
		name(true_outcomes_`i', replace)
	graph export "true_outcomes_`i'.png", name(true_outcomes_`i') replace
}
// make the censored outcome
gen ystar = y
replace ystar = ${h} if y >= ${h}

// If we knew the true outcome values, we could fit a multilevel regression:
mixed y c.week##i.train i.task || id: week, covariance(unstructured)
margins train, at(week = (1(1)5))
marginsplot, name(model1, replace) scheme(s1mono) ylabel(,grid) ytitle("") title("Model 1")

// If we ignore the censoring, we get biased results:
mixed ystar c.week##i.train i.task || id: week, covariance(unstructured)
margins train, at(week = (1(1)5))
marginsplot, name(model2, replace) scheme(s1mono) ylabel(,grid) ytitle("") title("Model 2")

// with multilevel Tobit, we can try to recover the missing information
metobit ystar c.week##i.train i.task || id: week, ul(30) covariance(unstructured)
margins train, at(week = (1(1)5))
marginsplot, name(model3, replace) scheme(s1mono) ylabel(,grid) ytitle("") title("Model 3")

graph combine model1 model2 model3, rows(1) scheme(s1mono) name(models, replace)
graph export models.svg, replace name(models)

